
# q-expansion of newform 1225.4.a.v, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1225
weight = 4
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[3, 1], [-1, 3], [0, 0], [0, 0], [14, -10], [-18, -10], [-38, -54], [-80, 26], [11, -117], [-125, 60], [66, -100], [208, 136], [53, 30], [333, 5], [-98, 64], [476, 142], [-420, -266], [-49, 570], [643, 69], [532, -350], [-86, 118], [-620, 214], [-953, 5], [-325, -324], [-314, 500], [-709, 180], [-1251, 387], [-617, -239], [535, -174], [-332, -650], [194, 1240], [296, -960], [868, 404], [-450, -150], [975, -1246], [-2206, 960], [1102, -104], [986, -692], [991, 1365], [-3336, -448], [-1270, 116], [-1297, 1000], [-1246, 1590], [-814, -1728], [-326, 260], [700, -2196], [292, 650], [1082, 2180], [382, 1396], [-210, -2924], [-2314, -102], [690, -260], [1306, 2620], [-62, -2740], [-68, -2516], [-3169, 2797], [-1815, -1586], [-5544, 1870], [-562, -3346], [1822, 380], [954, -2712], [3872, -2200], [-3519, -1025], [1456, 2530], [2564, 732], [-4342, 566], [-6566, -560], [1404, 650], [6623, -831], [945, 200], [4304, -1642], [-910, -7996], [7947, 4031], [2676, 2028], [-2600, 1390], [3589, 1557], [3050, -1084], [-8418, 384], [259, -8040], [3195, 5314], [-8220, -280], [6817, -90], [4714, 1030], [-1428, 10910], [-10080, -1504], [4241, -3867], [-6455, 3750], [-10732, -5644], [3098, 40], [-4737, 485], [5397, 7739], [-1830, -496], [1818, 4684], [-13838, -980], [-14210, -1706], [-7303, -7215], [-10935, -1024], [17666, -1900], [6474, -1948], [-471, 160], [10359, 1645], [4238, 12604], [12539, -3307], [1250, 13924], [-8956, 1290], [-14298, -944], [-17794, -1140], [-7056, -5518], [-3340, -5084], [-6702, -3460], [-11373, -8421], [-3674, 7822], [-5136, 3160], [9010, 294], [6622, 15150], [6959, 11490], [-3078, -860], [6067, -16589], [-9834, -3282], [-5560, 4370], [-7449, 950], [-7032, 16500], [-11118, 4454], [-319, 6017], [-5554, 3590], [4877, -17850], [13415, 6364], [4510, 15306], [-8469, -3675], [14754, 7272], [390, 2026], [-437, 6225], [-2556, -5380], [14374, -16550], [12466, 14120], [-790, 16560], [5114, -1702], [-20603, -3169], [-12424, 14000], [-11115, 9496], [-28542, -6900], [11014, -5000], [-12969, -3913], [-7261, 13595], [27670, 5864], [12410, 5490], [28932, -2570], [19702, 13836], [12440, 1456], [-469, 17593], [13788, -6174], [-407, -9070], [29918, -11380], [-32953, -3441], [14703, -11141], [-9498, 12270], [-16580, 15214], [-10325, 18406], [-2964, -28880], [-29094, 17700], [21603, -11839], [17048, 1380], [12889, 14005], [-16664, 16160], [18358, -14496], [8149, 6123], [-7486, 20320], [-29248, -5934], [-5755, -32480], [-23956, 19902], [-8340, 3516], [-5977, 4560], [16054, 33940], [-20064, -958], [-8890, -30976], [-15565, -15060], [-3708, -5070], [-9066, -1680], [32397, 545], [5905, 28926], [-11887, 31971], [-2362, 19820], [21058, -20920], [9882, -5376], [-23234, -8628], [-5595, 11204], [-11552, 13544], [-64106, 7388], [-26855, -16666], [-53934, 4850], [-8476, -5188], [5738, 22820], [3734, 4990], [20951, -37730], [-37722, 1444], [59214, -4872], [-48221, 570], [-66134, -6312], [-24444, -38410], [-9926, 2452], [2035, -10164], [-47642, 7760], [15752, -32606], [7955, -17096], [-62190, 11180], [-67668, -5914], [34360, -22374], [-3874, -47452], [10325, 7700], [-8584, -18340], [-5852, -25174], [35343, 35690], [-45137, -17155], [3977, -33731], [-23750, 44374], [-19614, 14840], [8053, 14259], [27361, 6720], [-48182, -20004], [34808, 6980], [-66571, -12210], [12330, 10], [13025, -28396], [29666, 1168], [-2699, 44525], [25755, 42130], [57944, 4112], [33970, 7126], [27067, 15489], [3134, 3060], [-14634, -46918], [86200, -3766], [17622, 14810], [32419, -18930], [25707, -51025], [-52033, 42491], [5335, -45014], [-314, 1478], [-22640, 25430], [22858, 6010], [-33994, -22028], [-46844, 9940], [51714, -23232], [-30305, -11016], [30432, -5820], [31970, 14986], [41431, 24735], [27306, 12920], [-60580, -28364], [-73722, -29060], [68784, -8000], [-15659, 16240], [1893, -40031], [84835, -2470], [81934, -5942], [82700, -16946], [46949, -60], [11397, -61539], [-57494, 30640], [16932, 44494], [-8669, -32583], [62239, 28005], [-36450, -20616], [14782, 59910], [-19088, 6876], [60750, 1086], [43790, -20560], [10498, -26080], [3153, -21775], [49956, -58168], [29731, -44280], [44338, 53224], [2564, 72258], [28160, 1764], [-54094, 17240], [26154, -25568], [50578, 55736], [-81985, -21044], [-29861, -44990], [-18924, -81210], [12189, -43347], [-39036, -71280], [39486, -72800], [93863, 22140], [50017, -36691], [60784, -63920], [41936, 64462], [-8816, 6600], [-68550, 55844], [118590, -10000], [509, 86610], [14097, 91391], [-91734, 38678], [-78042, 28610], [-16572, 16950], [-2915, -806], [151796, 9330], [15072, -78010], [-45050, 30486], [80926, -5820], [121734, -2938], [106228, -22634], [-79770, 24806], [6573, 18765], [-18186, 54720], [6514, 24530], [-113022, 33384], [97145, -37680], [96350, -4376], [51906, 56958], [-78171, 23567], [31110, -41944], [-44401, -58760], [34201, 71757], [-54311, -14265], [26255, 37364], [-101120, 29880], [-96296, 2130], [158682, -3180], [-110955, 28050], [9146, 57040], [58418, -11474], [11703, 47290], [107723, 46195], [83706, 43102], [93001, 30470], [-72970, 6344], [-135161, 36537], [-85702, -73464], [-10192, -73330], [-123186, 23200], [-9712, -5196], [29040, -17470], [-89241, -81757], [42754, 7430], [119521, 4775], [49910, -84360], [-167126, 14812], [90553, 10890], [82242, 43864], [83616, -42588], [57304, 18940], [-69335, -55650], [-26848, -102510], [-179654, -22932], [30640, -57190], [150131, -21070], [-29037, -63699], [58538, -96460], [38812, -46996], [20006, -35170], [-94042, -57866], [178317, -550], [-101151, 1883], [24935, 29264], [-95168, 59680], [16050, 59376], [17666, 72510], [-101462, -41954], [-112317, -74905], [-178596, 6970], [-57854, -24740], [-68827, -25641], [-82760, 13844], [165117, 20141], [-39074, 140938], [-118334, 27530], [-66694, -16048], [103767, -8830], [20784, -29110], [70390, 37336], [-69249, 44373], [-98350, -103060], [-143044, -23580], [-39922, 114066], [-67260, -96786], [191342, -3770], [94934, 11478], [-44830, -143464], [-5644, -63730], [-108869, -21640], [81398, 13910], [52689, -109285], [-13752, 15964], [112970, -85440], [-57371, 112383], [-18066, 48790], [-99368, -103244], [114501, -71547], [-44969, 54765], [105070, 47000], [-96936, -8908], [4270, -92444], [133382, -57876], [-50886, 42710], [-178084, -8618], [48980, -124286], [-6245, 96440], [-203628, -47220], [-17171, 136070], [-37555, 71766], [-45414, 44768], [45293, -33279], [-3508, 66244], [59265, 25994], [51696, -68910], [-136332, 18804], [-106573, -63280], [-208221, 9463], [-78490, 46356], [121644, 67502], [91598, 36816], [110638, 24600], [17004, -57220], [-163166, 61400], [-18974, -16580], [-86210, 24714], [60422, 16216], [17166, 116620], [58429, -115193], [19475, -60974], [135452, 61384], [-132031, 69765], [43470, 37040], [25722, -81700], [-35838, -58064], [85566, 36368], [-42915, 76650], [-70104, -73080], [153890, -54626]]
